import logging
import os
import sys
import time
from datetime import datetime as dt
import re

import click
import yaml
from feuersoftware import PublicAPI

from alarmparser import AlarmParser
from mailcheck import Mail
from telegram import telegram_bot
#Zur Nutzung des Telegram-Bots müssen nur die Zeilen 42,69,128 auskommentiert werden, also die Raute entfernen.

LOGLEVEL = "DEBUG"
#format = "%(asctime)s.%(msecs)03d - %(name)s [%(filename)s:%(lineno)s - %(funcName)20s()] %(levelname)s - %(message)s"
format = "%(asctime)s.%(msecs)03d - %(name)s - %(levelname)s - %(message)s"
dateformat = "%Y-%m-%d %H:%M:%S"

logging.basicConfig(level=LOGLEVEL, format=format, datefmt=dateformat)
LOGGER = logging.getLogger(">>>main<<<")
hdlr = logging.FileHandler('z_Logging.log')
hdlr.setLevel(level=logging.INFO)
formatter = logging.Formatter('%(asctime)s.%(msecs)03d - %(name)s [%(filename)s:%(lineno)s - %(funcName)20s()] %(levelname)s - %(message)s')
hdlr.setFormatter(formatter)
LOGGER.addHandler(hdlr)

class Observer:
    def __init__(self, config):
        self.config = config

    def run(self):
        while True:
            try:
                for account in self.config.get("accounts"):
                    try:
                        self.check_mail(account)
                    except:
                        postfachname = account.get("name")
                        LOGGER.error(f"Fehler in Observer {postfachname} aufgetreten - probiere weiter in 1 Sekunde")
                        telegram_bot.telegram_bot_sendtext(f"ERROR - Fehler in Observer {postfachname} aufgetreten")
                        time.sleep(1)
            except KeyboardInterrupt:
                LOGGER.info("Ctrl + C received. Stopping now!")
                sys.exit(0)
            time.sleep(4)

    def check_mail(self, account):
        """Check a mail account for new alarm mails."""
        _start = dt.now()
        name = account.get("name")
        mailcfg = account.get("mail")
        mail = Mail(mailcfg.get("host"), mailcfg.get("user"), mailcfg.get("password"))
        msgids = mail.search_mails(mailcfg.get("subject"))
        text = mail.get_text(msgids)
        if not text:
            return
        _fetchtime = dt.now()
        LOGGER.info(f"Fetching the alarmmail from {mailcfg.get('user')} took {(_fetchtime - _start).total_seconds()} seconds")
        data = AlarmParser(self.config, text).data
        _parsetime = dt.now()
        LOGGER.info(f"Parsing the alarmmail took {(_parsetime - _fetchtime).total_seconds()} seconds")
        alarmdata = self.assign_data(data)
        self.alert(account, alarmdata)
        _alerttime = dt.now()
        LOGGER.info(f"Sending to Connect API took {(_alerttime - _parsetime).total_seconds()} seconds")
        LOGGER.info(f"Entire process took {(_alerttime - _start).total_seconds()} seconds")
        #telegram_bot.telegram_bot_sendtext(f"ALARM - {name}")
      
    def assign_data(self, data):
              
        #Falls der Einsatzort eine Kreuzung ist, schreibe alles in eine Zeile
        streetandcross = "0"
        street2 = "0"
        cross = "0"
        street2 = data.get('fulladdress')
        cross = "" if data.get('crossstreet') == None else data.get("crossstreet")
        streetandcross = str(street2) + str(cross)
 
        #Wenn Ortsteil und Ort gleich, dann leere Variable erzeugen
        districtnew = "0"
        districtnew = "" if data.get('city') == data.get('district') else data.get('district')

        #Wenn kein Ort übermittelt, probiere es mit der Adresse allein
        citynew = "0"
        citynew = data.get('fulladdress') if data.get('city') == "" else data.get('city')
        
        #Wenn noch immer nichts da ist, dann schreibe das auf
        citynew = "keine Adresse übermittelt - Koordinaten beachten" if citynew == None else citynew
        
        #Wenn nichtmal die Koordinaten übergeben wurden, dann spinge ins Meer
        latnew = "0"
        longnew = "0"
        latnew = "54.254946" if data.get('latitude') == None else data.get('latitude')
        longnew = "7.020208" if data.get('longitude') == None else data.get('longitude')

        #Wir suchen das passende Stichwort
        rdkeyword = data.get('rdkeyword')
        fwkeyword = data.get('fwkeyword')
        textkeyword = data.get('textkeyword')
        keywordnew = None
                
        if rdkeyword is not None:
          keywordnew = rdkeyword
        if fwkeyword is not None:
          keywordnew = fwkeyword
        if keywordnew is None:
          keywordnew = textkeyword

        #Beim Stichwort MANV wird der Code im Keyword überschrieben (Achtung es gibt das Stichwort MANV auch, dann ist die Kategorie leer)
        
        manvkeyword = data.get("kategorie")
        
        if manvkeyword is None:
         manvkeyword = "0"
        
        if re.search(r'MANV\d*', manvkeyword):
         keywordnew = manvkeyword
        
        #Nun bauen wir die AMPDS-Info, wenn vorhanden
        ampdsrd = data.get('AMPDSTextRD')
        ampdsfw = data.get('AMPDSTextFW')
        AMPDSInfo = None
                
        if ampdsrd is not None:
          AMPDSInfo = ampdsrd
        if ampdsfw is not None:
          AMPDSInfo = ampdsfw

        
        #Wir basteln uns mal die alarmierten Kräfte
        #Aus der Liste der RICs wird eine Python-Liste        
        alarmiert = []
        alarmiert = data.get('ric')
        einzeldaten = []
        einzeldaten = alarmiert.split(",")
        
        #Bei mehreren Fahrzeugen ist da eine Leerzeile zuviel
        bereinigte_einzeldaten = [item.strip() for item in einzeldaten]
        
        #Aus der Liste wird die Datenzeile
        aaolist = []
        for item in bereinigte_einzeldaten:
           entry = {"Name": item, "RadioId": item}
           aaolist.append(entry)
                       
        """Assign data to right api fields, transform values if necessary."""
        properties = []

        if data.get('object') is not None:
         properties.append({"key": "Objekt:", "value": data.get('object')})

        if AMPDSInfo is not None:
         properties.append({"key": "AMPDS-Info:", "value": AMPDSInfo})

        if data.get('kategorie') is not None:
         properties.append({"key": "Kategorie:", "value": data.get('kategorie')})

        if data.get('AMPDS') is not None:
         properties.append({"key": "übermittelter Code:", "value": data.get('AMPDS')})

        if data.get('alarmnow') is not None:
         properties.append({"key": "z-aktuell Alarmiert:", "value": data.get('alarmnow')})

        alarmdata = {
            "start": dt.now().strftime('%Y-%m-%dT%H:%M:%S'),
            #"start": outputdatetime,
            "keyword": keywordnew,
            "number": data.get("number"),
            "address": {"Street": streetandcross, "City": citynew, "District": districtnew},
            "facts": data.get("facts"),
            "properties": properties,
           "position": {"latitude": latnew, "longitude": longnew},
           "ric": data.get('ric'),
           "assignedvehicles": aaolist
        } 
        LOGGER.info(alarmdata)
        return alarmdata

    def alert(self, account, alarmdata):
        """Send alarm data to Feuersoftware API."""
        api = PublicAPI(account.get("connect").get("token"))
        r = api.post_operation(
            start=alarmdata.get("start"),
            keyword=alarmdata.get("keyword"),
            status="update",
            alarmenabled=True,
            address=alarmdata.get("address"),
            facts=alarmdata.get("facts"),
            number=alarmdata.get("number"),
            properties=alarmdata.get("properties"),
            position=alarmdata.get("position"),
            ric=alarmdata.get("ric"),
            assignedvehicles=alarmdata.get("assignedvehicles"),
            source="AlarmPi"
            )


@click.command()
@click.argument("configfile", type=click.Path(exists=True))
def main(configfile):
    LOGGER.info("Programm wurde gestartet")
            
    with open(configfile) as cfg:
        config = yaml.load(cfg, Loader=yaml.BaseLoader)

    if not config:
        LOGGER.error("No config file found at %s!", configfile)
        sys.exit(1)

    
    observerloop(config)

def observerloop(config):
    while True:
        try:
            observer = Observer(config)
            observer.run()
            
        except: # catch *all* exceptions
            LOGGER.error("Fehler in observerloop aufgetreten - starte erneut in 10 Sekunden")
            telegram_bot.telegram_bot_sendtext("ERROR - Fehler in observerloop aufgetreten")
            time.sleep(10)



if __name__ == "__main__":
    main()
